<?php

namespace FiloBlu\Cegid\Helper\Cegid;

use FiloBlu\Cegid\Helper\Cegid\Order;
use RuntimeException;
use FiloBlu\CegidY2\Api\SaleDocumentService\EnumType\SaleDocumentType;
use FiloBlu\CegidY2\Api\SaleDocumentService\ServiceType\Create;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_Request;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\OmniChannel;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\SaleDocumentIdentifier;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\SaleDocumentReference;
use Magento\Rma\Api\RmaRepositoryInterface;
use Magento\Sales\Api\Data\OrderInterface;
use FiloBlu\CegidY2\Api\SaleDocumentService\ArrayType\ArrayOfUserDefinedDate;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\UserDefinedDate;
use Magento\Framework\App\ResourceConnection;

/**
 * Class OrderReturn
 * @package FiloBlu\Cegid\Helper\Cegid
 */
class OrderReturn extends Order {

    public function createReturn($rma_object){
        $this->is_return = true;
        $order = $this->current_order = $rma_object->getOrder();
        $this->orderCegidInit($order);

        $sku_qty_array_to_return = array();
        $items = $rma_object->getItems();
        foreach ($items as $item) {
            if ($item->getQtyApproved() > 0){
                $sku_qty_array_to_return[$item->getProductSku()] = $item->getQtyApproved();
            }
        }

        // The current source code in this case depends on the products that we are trying to return
        // Since with a multistock order we split the calls to CEGID (ex. ORDER1-SOURCE1, ORDER1-SOURCE2...)
        // Now we need to backtrace which order was
        $this->current_source_code = $this->findOriginalSourceBySku(array_keys($sku_qty_array_to_return),$rma_object);

        if (empty($sku_qty_array_to_return)){
            // No approved RMA quantities, nothing to send
            return false;
        }

        // Get base header
        $header = $this->getHeader($order);
        // Change some parameters because this is a return
        $header->setLinesUnmodifiable(false);
        $omnichannel = $this->getReturnOmniChannel($order);
        $header->setOmniChannel($omnichannel);
        $header->setType('ReturnNotice');

        // Set return as already accepted
        $header->setUserDefinedDates($this->getUserDefinedDates($rma_object));

        $lines = $this->getReturnLines($order,$sku_qty_array_to_return);

        /** @var Create_Request $request */
        $request = $this->createRequestFactory->create([
            'header' => $header
        ]);

        $request->setLines($lines);

        /** @var \FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create $parameters */
        $parameters = $this->createFactory->create();
        $parameters->setCreateRequest($request);
        $parameters->setClientContext($this->getRetailContext());

        // Execute
        $createService = $this->createService->create();
        $createService->Create($parameters);
        // Save data
        $request = $createService->getLastRequest();
        $response = $createService->getLastResponse();
        $error = $createService->getLastError();
        $result = $createService->getResult();

        if(!$result && $error){
            return $error;
        }
        return true;

        //throw new RuntimeException(implode("\n", $array_of_error_messages));
    }

    public function getUserDefinedDates($rma_object){
        $rma_comments = $rma_object->getComments();
        $last_rma_comment = end($rma_comments);
        $lastCommentDate = $last_rma_comment->getCreatedAt();
        $date = new \DateTime($lastCommentDate);

        $userDefinedDate = new UserDefinedDate();
        $userDefinedDate->setId('1');
        $userDefinedDate->setValue($date->format('Y-m-d'));

        $userDefinedDates = new ArrayOfUserDefinedDate();
        $userDefinedDates->addToUserDefinedDate($userDefinedDate);

        return $userDefinedDates;
    }

    public function getReturnOmniChannel(OrderInterface $order){
        $omnichannel = new OmniChannel();
        $omnichannel->setOriginalDocument($this->getReturnOmniChannelOriginalDocument($order));
        $omnichannel->setBillingStatus('NotPerformed');
        $omnichannel->setFollowUpStatus('WaitingForReturn');
        // TODO: payment method?
        //$omnichannel->setPaymentMethodId($this->getCegidPaymentMethodId($order));
        $omnichannel->setPaymentMethodId("RMB");
        $omnichannel->setReturnStatus('ReturnPlanned');
        $omnichannel->setReturnType('Return');
        $omnichannel->setShippingStatus('Pending');
        return $omnichannel;
    }

    public function getReturnOmniChannelOriginalDocument(OrderInterface $order){
        $reference = new SaleDocumentReference();
        $reference->setCustomerId($this->getCustomerCegidFromOrder($order));
        $reference->setInternalReference($this->getOrderReference($order,$this->current_source_code));
        $reference->setType(SaleDocumentType::VALUE_CUSTOMER_ORDER);
        $originalDocument = new SaleDocumentIdentifier();
        $originalDocument->setReference($reference);
        return $originalDocument;
    }


    public function getReturnLines($order,$sku_qty_array_to_return){
        $lines = $this->arrayOfCreateLineFactory->create();
        $origin = $this->getConfigSaleDocumentOrigin();

        foreach ($order->getItems() as $item) {

            if($item->getProductType() != 'configurable'){
                continue;
            }
            // If this item is one of those that i need to return go on
            if(!in_array($item->getSku(),array_keys($sku_qty_array_to_return))){
                continue;
            }

            // I need to split the grand total for each source, and everything must be in EUR
            // This MUST BE with VAT
            $single_unit_row_total = ($item->getBaseRowTotalInclTax() - $item->getBaseDiscountAmount()) / $item->getQtyOrdered();
            if ($order->getBaseCurrencyCode() !== self::EUR_CODE){
                $single_unit_row_total *= $order->getBaseToGlobalRate();
            }

            $line = $this->createLineFactory->create();
            $barcode = $item->getProduct()->getBarcode();
            $qty_to_take = $sku_qty_array_to_return[$item->getSku()];

            $this_order_row_total = $single_unit_row_total * $qty_to_take;
            $this->current_order_to_cegid_grandtotal += $this_order_row_total;

            $identifier = $this->itemIdentifierFactory->create(['reference' => $barcode]);
            $line->setItemIdentifier($identifier);
            $line->setOrigin($origin);
            $line->setQuantity($qty_to_take); //Not $item->getQtyOrdered()

            // Importo Iva a default articolo. In questo caso non va passata l’intera sezione Taxes
            //$line->setTaxes($this->getTaxes($item)); // This must be before the unit price

            // Importo unitario (UnitPrice) e Importo Netto Sconti (Net Unit Price) IVA ESCLUSA.
            $net_unit_price = $item->getBaseRowTotal() / $item->getQtyOrdered();
            $unit_price = ($item->getBaseRowTotal() - $item->getBaseDiscountAmount()) / $item->getQtyOrdered();
            if ($order->getBaseCurrencyCode() !== self::EUR_CODE){
                $net_unit_price *= $order->getBaseToGlobalRate();
                $unit_price *= $order->getBaseToGlobalRate();
            }
            $line->setNetUnitPrice($net_unit_price);
            // prezzo IVA ESCLUSA SENZA SCONTI  di una unità
            $line->setUnitPrice($unit_price);
            //$omniChannelLine = $this->getCegidReturnOmniChannelLine($this->current_source_code);
            //$line->setOmniChannel($omniChannelLine);
            $lines->addToCreate_Line($line);
        }
        return $lines;
    }

    public function getCegidReturnOmniChannelLine($warehouse_code)
    {
        $third_party = $this->sourceSlotsSourcesData->getThirdPartyCodeFromSourceCode($warehouse_code);
        $omniChannelLine = $this->omniChannelLineFactory->create();
        $omniChannelLine->setWarehouseId($third_party);
        return $omniChannelLine;
    }

    public function findOriginalSourceBySku($array_of_sku_to_rma,$rma_object){
        //TODO: Currently we can't backtrace RMA of a SKU where his quantity was splitted over multiple warehouses
        $array_of_source_codes = array();

        foreach ($array_of_sku_to_rma as $sku) {
            $sql = "SELECT IFBOS.inventory_source_id
                FROM magento_rma MR
                LEFT JOIN magento_rma_item_entity MRIE ON MR.entity_id = MRIE.rma_entity_id
                LEFT JOIN sales_order SO ON SO.entity_id = MR.order_id
                LEFT JOIN inventory_fb_orderitem_source IFBOS ON IFBOS.order_id = SO.entity_id
                LEFT JOIN sales_order_item SOI ON SOI.item_id = IFBOS.order_item_id
                WHERE MR.entity_id = {$rma_object->getId()} AND MRIE.status = 'approved' AND MRIE.product_sku = '{$sku}' LIMIT 1";
            $array_of_source_codes[] = $this->getConnection()->fetchOne($sql);
        }

        $array_of_source_codes = array_values(array_filter(array_unique($array_of_source_codes)));

        return $array_of_source_codes[0];
    }

}